// Helper Functions ------------------------------------------
// -----------------------------------------------------------

void update_status(char *newStatus) {

	gtk_label_set(GTK_LABEL(status_bar), newStatus);
	while (gtk_events_pending())
		gtk_main_iteration();
}

void error_dialog(char *Error) {

	GtkWidget *dialog, *label, *button;

	dialog = gtk_dialog_new();
	label = gtk_label_new(Error);
	button = gtk_button_new_with_label("OK");
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);     
	gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
                            GTK_OBJECT(dialog));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
			   label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
			   button, TRUE, TRUE, 0);
	gtk_widget_show(label);
	gtk_widget_show(button);
	gtk_widget_show(dialog);
	gtk_widget_grab_default (button);
}

// Camera Functions ------------------------------------------
// -----------------------------------------------------------

int open_camera () {

	long ltemp;

        if (eph_open(iob, serial_port, 115200) == -1)
		return (0);
	sleep(1);
        eph_getint(iob, 35, &ltemp);
	return (1);
}

void close_camera() {

	eph_close(iob, 0);
}

struct ImageInfo *grabimage (long picNum, int thumbnail) {

        long thumbLength, picLength, Size;
        static FILE *fp = NULL;
        char *picData;
	char tempName[1024], rm[1024];
        long picSize, ltemp;
	int pid;

	struct ImageInfo *ret;
	GdkPixmap *pixmap;
	GtkWidget *gpixmap;
	gint w,h;

	ret = (struct ImageInfo *) malloc (sizeof(struct ImageInfo));

	open_camera();
		
        eph_setint(iob, 4, picNum);
        eph_getint(iob, 0x0d, &thumbLength);
        eph_getint(iob, 0x0c, &picLength);

	if (thumbnail)
		Size = thumbLength;
	   else 
		Size = thumbLength + picLength;
        Size = ((Size-1)/2048+2)*2048;

        picData = malloc(Size);

	if (thumbnail)
	        eph_getvar(iob, 0x0f, &picData, &picSize);
	   else
	        eph_getvar(iob, 0x0e, &picData, &picSize);

	close_camera();

	pid = getpid();
	if (thumbnail)
		sprintf(tempName, "%s/gphoto-thumbnail-%li.jpg",
			gphotoDir, picNum);
	   else {
		sprintf(tempName, "%s/gphoto-%i-%li.jpg", 
			gphotoDir, pid, picCounter);
		picCounter++;
	}
        fp = fopen(tempName, "w");
        fwrite(picData, (size_t)Size, 1, fp);
        fclose(fp);

	ret->imlibimage = gdk_imlib_load_image(tempName);
	w = ret->imlibimage->rgb_width; h = ret->imlibimage->rgb_height;
	gdk_imlib_render(ret->imlibimage, w, h);
	pixmap = gdk_imlib_move_image(ret->imlibimage);
	ret->image = gtk_pixmap_new(pixmap, NULL);

	sprintf(rm, "rm %s", tempName);
	system(rm);
	return (ret);
}

void appendpic (long picNum) {

/* -- this is for testing when i don't have my digicam with me
	fp = fopen ("test.jpg", "r");
	fseek(fp, 0, SEEK_END);
	numRead = ftell(fp);
	rewind(fp);
	picData = malloc(numRead);
	fread(picData, (size_t)numRead, 1, fp);
	fclose (fp);
*/

	int i=0;
	char fname[15];
	
	GtkWidget *scrwin, *label;

	while (Images[i] != NULL)
		i++;

	Images[i] = (struct ImageInfo *) malloc(sizeof(struct ImageInfo));

	Images[i] = grabimage(picNum, 0);
	gtk_widget_show(Images[i]->image);

	// Append to notebook --------------------------------
	sprintf(fname, "Image %li", picNum);
        Images[i]->page = gtk_vbox_new(FALSE,0);
	gtk_widget_show(Images[i]->page);
	label = gtk_label_new(fname);
        gtk_notebook_append_page (GTK_NOTEBOOK(notebook), Images[i]->page,
                                  label);
        scrwin = gtk_scrolled_window_new(NULL,NULL);
        gtk_widget_show(scrwin);
	gtk_container_add(GTK_CONTAINER(Images[i]->page), scrwin);
	gtk_container_add(GTK_CONTAINER(scrwin), Images[i]->image);
	numPages++;
}

// GTK Functions ---------------------------------------------
// -----------------------------------------------------------

void destroy (GtkWidget *widget, gpointer data) {

        gtk_main_quit ();
}

gint delete_event (GtkWidget *widget, GdkEvent *event, gpointer data) {

        destroy(widget, data);
        return (FALSE);
}


// Callbacks -------------------------------------------------
// -----------------------------------------------------------

void save_port(GtkWidget *widget, GtkWidget *dialog) {

	FILE *conf;
	char gphotorc[1024];
	int port=0;	

	sprintf(gphotorc, "%s/gphotorc", gphotoDir);

	conf = fopen(gphotorc, "w");
	if (GTK_WIDGET_STATE(port0) == GTK_STATE_ACTIVE) {
		fwrite("0", (size_t)sizeof(char), (size_t)1, conf);
		port=0;
	}
	if (GTK_WIDGET_STATE(port1) == GTK_STATE_ACTIVE) {
		fwrite("1", (size_t)sizeof(char), (size_t)1, conf);
		port=1;
	}
	if (GTK_WIDGET_STATE(port2) == GTK_STATE_ACTIVE) {
		fwrite("2", (size_t)sizeof(char), (size_t)1, conf);
		port=2;
	}

	if (GTK_WIDGET_STATE(port3) == GTK_STATE_ACTIVE) {
		fwrite("3", (size_t)sizeof(char), (size_t)1, conf);
		port=3;
	}

	fclose(conf);
	sprintf(serial_port, "/dev/ttyS%c", port);
	gtk_widget_destroy(dialog);
}

void port_dialog() {

	GtkWidget *dialog, *label, *button;
	GSList *group;

	dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), "Select port...");
	// gtk_widget_set_usize(dialog, 460, 450);
	label = gtk_label_new("Please select a port:");
	gtk_widget_show(label);
	port0 = gtk_radio_button_new_with_label(NULL, "Serial Port 1");
        gtk_widget_show(port0);
	gtk_button_clicked(GTK_BUTTON(port0));
        group = gtk_radio_button_group(GTK_RADIO_BUTTON(port0));
        port1 = gtk_radio_button_new_with_label(group, "Serial Port 2");
        gtk_widget_show(port1);
        group = gtk_radio_button_group(GTK_RADIO_BUTTON(port1));
        port2 = gtk_radio_button_new_with_label(group, "Serial Port 3");
        gtk_widget_show(port2);
        group = gtk_radio_button_group(GTK_RADIO_BUTTON(port2));
        port3 = gtk_radio_button_new_with_label(group, "Serial Port 4");
        gtk_widget_show(port3);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
                           label, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
                           port0, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
                           port1, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
                           port2, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
                           port3, FALSE, FALSE, 0);
        button = gtk_button_new_with_label("Save");
        gtk_widget_show(button);
        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                            GTK_SIGNAL_FUNC(save_port),
                            GTK_OBJECT(dialog));
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
                           button, TRUE, TRUE, 0);
        gtk_widget_show(dialog);
        gtk_widget_grab_default(button);
}

void version_dialog() {

	error_dialog("
Version Information
-------------------

Current Version: 0.0.1

This is a pre-release of gPhoto. Due to several requests, we
decided to make a pre-release with basic functionality. Please
keep in mind that bugs probably do exist. See the README file 
that came with this distribution, and please submit bug reports
to the address listed there.

Thanx much. :)
");

}
	
void usersmanual_dialog() {

	GtkWidget *dialog, *label, *scrwin, *button;

	dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), "User's Manual");
	gtk_widget_set_usize(dialog, 460, 450);

	label = gtk_label_new("
gPhoto v0.0.1 User's Manual
---------------------------

gPhoto was designed to be as intuitive as possible. The toolbar provides
shortcuts to many of the features listed below.


Getting Thumbnails/Previews
---------------------------
	In the \"Camera\" menu, select \"Get Index\"

Downloading Images
------------------
	After getting the index, select the images you want to download
	by clicking on them (border will turn red). Then, in the
	\"Camera\" menu, select \"Get Selected Images\".

Saving Images
-------------
	Click on the tab of the image you want to save, and select
	\"Save\" from the \"File\" menu. Select the directory you want
	to save the image in, and type in a name for the image.
	The extension will determine which format the image is saved
	in.

	Example: image.jpg

	Supported: jpg gif png tif (and others...)
	(whatever is supported by your installation of imlib)

Printing Images
---------------
	Click on the tab of the image you want to print, and select
	\"Print\" from the \"File\" menu.
	Note: it might take a minute for the image to print, depending
	on how fast the image is spooled.

Closing Downloaded Images
-------------------------
	Click on the tab of the image you want to close, and select
	\"Close\" from the \"File\" menu.

Deleting Images from the Camera
-------------------------------
	In the image index, select the images you wish to remove from
	the camera by clicking on the thumnail (border will turn red).
	Then, from the \"Camera\" menu, select \"Delete Selected Images\".
	A confirmation will appear. Click \"yes\" or \"no\".

Configuring the Camera
----------------------
	Select \"Configure\" from the \"Camera\" menu.

	
Tips and Tricks
---------------
	* Make sure your camera is connected and turned ON. :-)
	* If something fails (getting index, configuring camera, ...)
	  try again. There are some things that need to be done still
	  to make sure the camera does what you want it to do.
	* ???
");
	gtk_widget_show(label);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
	scrwin = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrwin);
	gtk_container_add(GTK_CONTAINER(scrwin), label);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),scrwin);

	button = gtk_button_new_with_label("OK");
	gtk_widget_show(button);
        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                            GTK_SIGNAL_FUNC(gtk_widget_destroy),
                            GTK_OBJECT(dialog));
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
                           button, TRUE, TRUE, 0);	
	gtk_widget_show(dialog);
	gtk_widget_grab_default(button);
}

void faq_dialog() {

error_dialog("
Please visit http://gphoto.fix.no/gphoto/faq.shtml
for the current FAQ list.

For information on using gPhoto, see the \"User's Manual\"
in the \"Help\" menu.");
}

void about_dialog() {

error_dialog("
GNU Photo pre-release version 0.0.1

Copyright (C) 1998 Scott Fritzinger <scottf@scs.unr.edu>
Copyright (C) 1998 Ole Kristian Aamot <oleaa@ifi.uio.no>
GNU Photo is the ultimate free graphical application for digital
still cameras, based on GTK+.  It aims to be GNOME compliant.

GNU Photo uses the PhotoPC library (libeph_io).
Copyright (c) 1997,1998 Eugene G. Crosser <crosser@average.org>
Copyright (c) 1998 Bruce D. Lightner (DOS/Windows support)

You may distribute and/or use libeph_io for any purpose modified
or unmodified copies of the library, if you preserve the copyright
notice above.

GNU Photo is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for further details.
(http://www.gnu.org/copyleft/gpl.html) 

Visit http://gphoto.fix.no/gphoto/ for GNU Photo updates.");
}

// used until callback is implemented
void menu_selected (gchar *toPrint) {

	char Error[1024];
	
	sprintf(Error, "%s is not yet implemented", toPrint);
	error_dialog(Error);
}

void remove_thumbnail (int i) {

	struct ImageInfo *del = Thumbnails[i-1];
	
	while (Thumbnails[i] != NULL) {
		Thumbnails[i-1] = Thumbnails[i];
		i++;
	}

	if (del->imlibimage)
		gdk_imlib_kill_image(del->imlibimage);

	if (GTK_PIXMAP(del->image)->pixmap) {
		gdk_pixmap_unref(GTK_PIXMAP(del->image)->pixmap);
		gdk_imlib_free_pixmap(GTK_PIXMAP(del->image)->pixmap);	
	}

	gtk_widget_destroy(del->button);
	Thumbnails[i-1] = NULL;
}

// get index of images and place in main page table
void getindex () {

	int i, done=0;
	char c[10];
	char fname[1024], rm[1024], status[256];

	GtkStyle *style;
	GtkWidget *scrwin, *label, *gpixmap, *vbox, *picLabel;

	if (open_camera() == 0) {
		error_dialog("Could not open camera.");
		return;
	}
        eph_getint(iob, 0x0a, &num_pictures_taken);
	close_camera();

	update_status("Getting Index...");	

	if (Thumbnails[0] != NULL) {
		for (i=num_pictures_taken; i>=1; i--) {
			remove_thumbnail(i);
		}
	}
	if (index_table != NULL) {
		gtk_widget_destroy(index_table);
	}

	index_table = gtk_table_new(100,6,FALSE);
        gtk_widget_show(index_table);
        gtk_container_add(GTK_CONTAINER(index_window), index_table);

	for (i=1; i<=num_pictures_taken; i++) {
		sprintf(status, "Getting thumbnail %i...", i);
		update_status(status);
		Thumbnails[i-1] = grabimage(i, 1);
		Thumbnails[i-1]->button = gtk_toggle_button_new();
		gtk_widget_show(Thumbnails[i-1]->button);
		gtk_widget_show(Thumbnails[i-1]->image);
		style = gtk_style_copy(
			gtk_widget_get_style(Thumbnails[i-1]->button));
		style->bg[GTK_STATE_ACTIVE].green=0;
		style->bg[GTK_STATE_ACTIVE].blue=0;
		gtk_widget_set_style(Thumbnails[i-1]->button, style);

		vbox = gtk_vbox_new(FALSE, 0);
		gtk_container_border_width(GTK_CONTAINER(vbox),3);
		gtk_widget_show(vbox);

		gtk_container_add(
			GTK_CONTAINER(Thumbnails[i-1]->button),vbox);
		gtk_container_add(GTK_CONTAINER(vbox), 
				  Thumbnails[i-1]->image);

		gtk_table_attach(GTK_TABLE(index_table),
				Thumbnails[i-1]->button,
				 (i-1)%6,(i-1)%6+1,(i-1)/6,(i-1)/6+1,
				 GTK_FILL,GTK_FILL,5,5);

		while (gtk_events_pending())
			gtk_main_iteration();
	}
	update_status("Click on a thumbnail to mark for download, then select \"Camera->Get Selected Images\"");	
}

// get selected pictures
void getpics () {

	int something_selected = 0;
	char status[256];
	long i;

	for (i=1; i<=num_pictures_taken; i++) {
		if (GTK_TOGGLE_BUTTON(Thumbnails[i-1]->button)->active) {
			something_selected = 1;
			sprintf(status, "Getting image %i (this might take a minute)", i);
			update_status(status);
			gtk_toggle_button_set_state(
				GTK_TOGGLE_BUTTON(Thumbnails[i-1]->button),
				FALSE);
			appendpic(i);
			while (gtk_events_pending())
				gtk_main_iteration();
		}
	}
	if (something_selected)
		update_status("Done downloading. Click on the new tabs to view downloaded images");
	else
		update_status("Nothing selected for download.");
}

void remove_image(int i) {

	int x;

	struct ImageInfo *del = Images[i-1];

	while (Images[i] != NULL) {
		Images[i-1] = Images[i];
		i++;
	}
	Images[i] = NULL;
	gdk_imlib_kill_image(del->imlibimage);
	gdk_pixmap_unref(GTK_PIXMAP(del->image)->pixmap);
	gdk_imlib_free_pixmap(GTK_PIXMAP(del->image)->pixmap);
	gtk_widget_destroy(del->image);
	gtk_widget_destroy(del->page);
	free(del);
}

void closepic () {

	int currentPage;
	char rm[1024];

	currentPage = gtk_notebook_current_page (GTK_NOTEBOOK(notebook));

	if (currentPage != 0) {
		remove_image(currentPage);
	}
}

void savepic (GtkWidget *widget, GtkFileSelection *fwin) {

	int currentPic, i;
	char *fname, cp[1024];

	gtk_widget_hide(GTK_WIDGET(fwin));
	i = gtk_notebook_current_page (GTK_NOTEBOOK(notebook));

	if (currentPic != 0) {
		fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fwin));
		gdk_imlib_save_image(Images[i-1]->imlibimage, fname, NULL);
		update_status("Image Saved.");
	}
}

	// save picture being viewed as jpeg
void savedialog () {

	int currentPic;
	GtkWidget *filew;

	filew = gtk_file_selection_new ("Save Image...");
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION
			    (filew)->ok_button), "clicked",
			    (GtkSignalFunc) savepic,
			    filew);
        gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
			    (filew)->cancel_button), "clicked",
			    (GtkSignalFunc) gtk_widget_hide,
                            GTK_OBJECT (filew));
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew),"*.jpg");
	currentPic = gtk_notebook_current_page (GTK_NOTEBOOK(notebook));
	if (currentPic != 0)
	        gtk_widget_show(filew);
	   else
		error_dialog("Saving thumbnails is not yet supported.");

}

void send_to_gimp () {

	error_dialog("Send to Gimp not yet supported.");
}

void print_pic () {

	int currentPic, i, pid;
	char command[1024], fname[1024];

	currentPic = gtk_notebook_current_page (GTK_NOTEBOOK(notebook));
	if (currentPic == 0) {
		error_dialog("Can't print the index yet.");
		return;
	}
	pid = getpid();
	sprintf(fname, "%s/gphoto-%i-%i.jpg", gphotoDir, pid, currentPic);
	gdk_imlib_save_image(Images[currentPic-1]->imlibimage, fname,
			     NULL);
	update_status("Now spooling...");
	sprintf(command, "lpr -r %s", fname);
	system(command);
	update_status("Spooling done. Printing may take a minute.");

}

void save_config (GtkWidget *widget, GtkWidget *dialog) {

	long value;
	char *camID;
	GtkAdjustment *adj;
	time_t camtime;

	gtk_widget_hide(dialog);
	update_status("Saving Configuration...");

	if (open_camera() == 0) {
		error_dialog("Could not open camera.");
		return;
	}

	camID = gtk_entry_get_text(GTK_ENTRY(Config.cam_id));
	eph_setvar(iob,0x16,camID,strlen(camID));
	sleep(1);

	// Set image quality...
	if (GTK_WIDGET_STATE(Config.qual_std) == GTK_STATE_ACTIVE)
		value = 1;
	  else if (GTK_WIDGET_STATE(Config.qual_high) == GTK_STATE_ACTIVE)
		value = 2;
	  else
		value = 3;
	eph_setint(iob,1,value);
	sleep(1);

	// Set flash mode...
	if (GTK_WIDGET_STATE(Config.flash_auto) == GTK_STATE_ACTIVE)
		value = 0;
	 else if (GTK_WIDGET_STATE(Config.flash_force) == GTK_STATE_ACTIVE)
 		value = 1;
	 else if (GTK_WIDGET_STATE(Config.flash_none) == GTK_STATE_ACTIVE)
 		value = 2;
	 else 
 		value = 3;
	eph_setint(iob,7,value);
	sleep(1);

	// Set lens mode...
	if (GTK_WIDGET_STATE(Config.lens_mac) == GTK_STATE_ACTIVE)
		value = 1;
	   else
		value = 2;
	eph_setint(iob,33,value);
	sleep(1);

	// Set date format...
	if (GTK_WIDGET_STATE(Config.date_yymmdd) == GTK_STATE_ACTIVE)
		value = 1;
	   else
		value = 2;
	eph_setint(iob,41,value);
	sleep(1);

	// Set LCD brightness ...
	adj = gtk_range_get_adjustment(GTK_RANGE(Config.lcd));
	value = adj->value;
	eph_setint(iob,35,value);
	sleep(1);

	// Set power savings ...
	adj = gtk_range_get_adjustment(GTK_RANGE(Config.docked));
	value = adj->value;
	eph_setint(iob,23,value);
	adj = gtk_range_get_adjustment(GTK_RANGE(Config.undocked));
	value = adj->value;
	eph_setint(iob,24,value);
	sleep(1);

	// Set the clock...
	if (GTK_WIDGET_STATE(Config.clk_comp) == GTK_STATE_ACTIVE) {
		camtime = time(&camtime);
		eph_setint(iob,2,camtime);
		sleep(1);
	}
	close_camera();
	gtk_widget_destroy(dialog);
	update_status("Done.");
}

void config_dialog () {

	char *info;
	off_t info_size = 2048;
	time_t camtime;
	char *atime;
	char currtime[256];

	long value;

	GtkWidget *dialog, *table, *label, *spacer, *hbox;
	GtkWidget *save_button, *cancel_button;
	GtkObject *adj;
	GSList *group;

	info = malloc(2048);

	update_status("Getting Camera Configuration...");

	// dialog window ----------------------
	dialog = gtk_dialog_new();
	gtk_window_set_title (GTK_WINDOW(dialog), "Configure Camera");
	gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
				   10);
	table = gtk_table_new(15,5,FALSE);
	gtk_widget_show(table);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);

	spacer = gtk_vseparator_new();
	gtk_widget_show(spacer);
	gtk_table_attach_defaults(GTK_TABLE(table),spacer,2,3,2,15);	

	if (open_camera() == 0) {
		error_dialog("Could not open camera.");
		return;
	}

	// camera id ----------------------
	label = gtk_label_new("Camera ID:");
	gtk_widget_show(label);
	Config.cam_id = gtk_entry_new();
	gtk_widget_show(Config.cam_id);
	gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.cam_id,1,5,0,1);

	spacer = gtk_hseparator_new();
	gtk_widget_show(spacer);
	gtk_table_attach_defaults(GTK_TABLE(table),spacer,0,5,1,2);	

	info[0] = '\0'; info_size=2048;
	eph_getvar(iob,0x16,&info,&info_size);
	gtk_entry_set_text(GTK_ENTRY(Config.cam_id), info);

	// image quality ----------------------
	label = gtk_label_new("Image Quality:");
	gtk_widget_show(label);
	Config.qual_std = gtk_radio_button_new_with_label(NULL, "Standard");
	gtk_widget_show(Config.qual_std);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(Config.qual_std));
	Config.qual_high = gtk_radio_button_new_with_label(group, "High");
	gtk_widget_show(Config.qual_high);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(Config.qual_high));
	Config.qual_best = gtk_radio_button_new_with_label(group, "Best");
	gtk_widget_show(Config.qual_best);
	gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,2,3);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.qual_std,1,2,2,3);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.qual_high,1,2,3,4);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.qual_best,1,2,4,5);	

	spacer = gtk_hseparator_new();
	gtk_widget_show(spacer);
	gtk_table_attach_defaults(GTK_TABLE(table),spacer,0,2,5,6);	

	eph_getint(iob,1,&value);
	switch (value) {
		case 2:
			gtk_widget_activate(Config.qual_best);
			break;
		case 3:
			gtk_widget_activate(Config.qual_high);
			break;
		default:
			gtk_widget_activate(Config.qual_std);
	}
			

	// lcd brightness ----------------------
	label = gtk_label_new("LCD Brightness:");
	gtk_widget_show(label);
	eph_getint(iob,35,&value);
	adj = gtk_adjustment_new(value, 1, 7, 1, 0, 0);
	Config.lcd = gtk_hscale_new(GTK_ADJUSTMENT(adj));
	gtk_range_set_update_policy(GTK_RANGE(Config.lcd),GTK_UPDATE_CONTINUOUS);
	gtk_scale_set_draw_value(GTK_SCALE(Config.lcd), TRUE);
	gtk_scale_set_digits(GTK_SCALE(Config.lcd), 0);
	gtk_widget_show(Config.lcd);
	gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,6,7);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.lcd,0,2,7,8);	

	spacer = gtk_hseparator_new();
	gtk_widget_show(spacer);
	gtk_table_attach_defaults(GTK_TABLE(table),spacer,0,2,8,9);	

	// power save --------------------------
	label = gtk_label_new("Power Saving:");
	gtk_widget_show(label);
	gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,9,10);	
	label = gtk_label_new("Docked");
	gtk_widget_show(label);
	gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,10,11);	
	label = gtk_label_new("Undocked");
	gtk_widget_show(label);
	gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,11,12);	
	eph_getint(iob,23,&value);
	adj = gtk_adjustment_new(value, 0, 180, 5, 0, 0);
	Config.docked = gtk_hscale_new(GTK_ADJUSTMENT(adj));
	gtk_range_set_update_policy(GTK_RANGE(Config.docked),
				    GTK_UPDATE_CONTINUOUS);
	gtk_scale_set_draw_value(GTK_SCALE(Config.docked), TRUE);
	gtk_scale_set_digits(GTK_SCALE(Config.docked), 0);
	gtk_widget_show(Config.docked);
	eph_getint(iob,24,&value);
	adj = gtk_adjustment_new(value, 0, 180, 5, 0, 0);
	Config.undocked = gtk_hscale_new(GTK_ADJUSTMENT(adj));
	gtk_range_set_update_policy(GTK_RANGE(Config.undocked),
				    GTK_UPDATE_CONTINUOUS);
	gtk_scale_set_draw_value(GTK_SCALE(Config.undocked), TRUE);
	gtk_scale_set_digits(GTK_SCALE(Config.undocked), 0);
	gtk_widget_show(Config.undocked);
	gtk_table_attach_defaults(GTK_TABLE(table),Config.docked,0,1,10,11);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.undocked,0,1,11,12);	

	spacer = gtk_hseparator_new();
	gtk_widget_show(spacer);
	gtk_table_attach_defaults(GTK_TABLE(table),spacer,0,2,12,13);	

	// lens mode -----------------------
	label = gtk_label_new("Lens Mode:");
	gtk_widget_show(label);
	Config.lens_norm = gtk_radio_button_new_with_label(NULL, "Normal");
	gtk_widget_show(Config.lens_norm);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(Config.lens_norm));
	Config.lens_mac = gtk_radio_button_new_with_label(group, "Macro");
	gtk_widget_show(Config.lens_mac);
	gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,13,14);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.lens_norm,1,2,13,14);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.lens_mac,1,2,14,15);	
	
	eph_getint(iob,33,&value);
	switch (value) {
		case 1:
			gtk_widget_activate(Config.lens_mac);
			break;
		default:
			gtk_widget_activate(Config.lens_norm);
	}

	// flash mode ----------------------
	label = gtk_label_new("Flash Mode:");
	gtk_widget_show(label);
	Config.flash_auto = gtk_radio_button_new_with_label(NULL, "Auto");
	gtk_widget_show(Config.flash_auto);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(Config.flash_auto));
	Config.flash_red = gtk_radio_button_new_with_label(group, "Red-eye");
	gtk_widget_show(Config.flash_red);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(Config.flash_red));
	Config.flash_force = gtk_radio_button_new_with_label(group, "Force");
	gtk_widget_show(Config.flash_force);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(Config.flash_force));
	Config.flash_none = gtk_radio_button_new_with_label(group, "None");
	gtk_widget_show(Config.flash_none);
	gtk_table_attach_defaults(GTK_TABLE(table),label,3,4,2,3);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.flash_auto,4,5,2,3);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.flash_red,4,5,3,4);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.flash_force,4,5,4,5);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.flash_none,4,5,5,6);	
	
	spacer = gtk_hseparator_new();
	gtk_widget_show(spacer);
	gtk_table_attach_defaults(GTK_TABLE(table),spacer,3,5,6,7);	

	eph_getint(iob,7,&value);
	switch (value) {
		case 1:
			gtk_widget_activate(Config.flash_force);
			break;
		case 2:
			gtk_widget_activate(Config.flash_none);
			break;
		case 3:
			gtk_widget_activate(Config.flash_red);
			break;
		default:
			gtk_widget_activate(Config.flash_auto);
	}

	// date format -------------------------
	label = gtk_label_new("Date Format:");
	gtk_widget_show(label);
	Config.date_yymmdd = gtk_radio_button_new_with_label(NULL, "YY MM DD");
	gtk_widget_show(Config.date_yymmdd);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(Config.date_yymmdd));
	Config.date_ddmmhh = gtk_radio_button_new_with_label(group, "DD MM HH");
	gtk_widget_show(Config.date_ddmmhh);
	gtk_table_attach_defaults(GTK_TABLE(table),label,3,4,7,8);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.date_yymmdd,4,5,7,8);	
	gtk_table_attach_defaults(GTK_TABLE(table),Config.date_ddmmhh,4,5,8,9);	

	spacer = gtk_hseparator_new();
	gtk_widget_show(spacer);
	gtk_table_attach_defaults(GTK_TABLE(table),spacer,3,5,9,10);	
	
	eph_getint(iob,41,&value);
	switch (value) {
		case 2:
			gtk_widget_activate(Config.date_ddmmhh);
			break;
		default:
			gtk_widget_activate(Config.date_yymmdd);
	}	

	// set clock ----------------------------
	label = gtk_label_new("Current Camera Time:");
	gtk_widget_show(label);
	gtk_table_attach_defaults(GTK_TABLE(table),label,3,4,10,11);

	eph_getint(iob,2,&camtime);
	atime = ctime(&camtime);
	label = gtk_label_new(atime);
	gtk_widget_show(label);
	gtk_table_attach_defaults(GTK_TABLE(table),label,3,5,11,12);

	Config.clk_comp = gtk_radio_button_new_with_label(NULL,
							 "Set to Computer");
	gtk_widget_show(Config.clk_comp);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(Config.clk_comp));
	Config.clk_none = gtk_radio_button_new_with_label(group, "No Change");
	gtk_widget_show(Config.clk_none);

	gtk_table_attach_defaults(GTK_TABLE(table),Config.clk_comp,4,5,12,13);
	gtk_table_attach_defaults(GTK_TABLE(table),Config.clk_none,4,5,13,14);

	gtk_widget_activate(Config.clk_none);

	// WOW that was a lot of code... now to get the camera info ---
	// (note to self: clean up the above code... :P  )

	close_camera();

	save_button = gtk_button_new_with_label("Save");
	gtk_widget_show(save_button);
	gtk_signal_connect_object(GTK_OBJECT(save_button), "clicked", 
			          GTK_SIGNAL_FUNC(save_config),
				  GTK_OBJECT(dialog));
	GTK_WIDGET_SET_FLAGS (save_button, GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
			   save_button, FALSE, FALSE, 0);
	
	cancel_button = gtk_button_new_with_label("Cancel");
	gtk_widget_show(cancel_button);
	gtk_signal_connect_object(GTK_OBJECT(cancel_button), "clicked", 
				  GTK_SIGNAL_FUNC(gtk_widget_destroy), 
				  GTK_OBJECT(dialog));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
			   cancel_button, FALSE, FALSE, 0);

	gtk_widget_grab_default (save_button);
	gtk_widget_show(dialog);
	update_status("Done.");
}

void del_pics (GtkWidget *widget, GtkWidget *dialog) {

	int i=0, j=num_pictures_taken;
	char z=0;

	gtk_widget_hide(dialog);
	update_status("Deleting selected pictures...");

	if (open_camera() == 0) {
		error_dialog("Could not open camera.");
		return;
	}

	// delete from camera...
	for (i=1; i<=num_pictures_taken; i++) {
		if (GTK_TOGGLE_BUTTON(Thumbnails[i-1]->button)->active) {
			eph_setint(iob,4,i);
			sleep(2);
			eph_action(iob,7,&z,1);
			remove_thumbnail(i);
			i--;
			num_pictures_taken--;
		}
	}
	close_camera();
	gtk_widget_destroy(dialog);
	update_status("Done.");
}

void del_dialog () {

	GtkWidget *dialog, *button, *label;

	dialog = gtk_dialog_new();

	label = gtk_label_new("Are you sure you want to DELETE the selected images?");
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label,
			   FALSE, FALSE, 0);

	button = gtk_button_new_with_label("Yes");
	gtk_widget_show(button);
	gtk_signal_connect_object(GTK_OBJECT(button), "clicked", 
				  GTK_SIGNAL_FUNC(del_pics), 
				  GTK_OBJECT(dialog));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
			   button, FALSE, FALSE, 0);
	button = gtk_button_new_with_label("No");
	gtk_widget_show(button);
	gtk_signal_connect_object(GTK_OBJECT(button), "clicked", 
				  GTK_SIGNAL_FUNC(gtk_widget_destroy), 
				  GTK_OBJECT(dialog));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
			   button, FALSE, FALSE, 0);

	gtk_widget_show(dialog);
}

void select_all() {

	int i;

	for (i=1; i<=num_pictures_taken;i++) {
		if (!GTK_TOGGLE_BUTTON(Thumbnails[i-1]->button)->active)
		gtk_button_clicked(GTK_BUTTON(Thumbnails[i-1]->button));
	}
}

void select_none() {

	int i;

	for (i=1; i<=num_pictures_taken;i++) {
		if (GTK_TOGGLE_BUTTON(Thumbnails[i-1]->button)->active)
		gtk_button_clicked(GTK_BUTTON(Thumbnails[i-1]->button));
	}
}
